package geniusweb.bidspace;

import static org.junit.Assert.assertEquals;

import java.io.IOException;
import java.math.BigInteger;
import java.util.Arrays;
import java.util.Collection;
import java.util.Random;

import org.junit.Test;

import com.fasterxml.jackson.core.JsonParseException;
import com.fasterxml.jackson.databind.JsonMappingException;
import com.fasterxml.jackson.databind.ObjectMapper;

import geniusweb.issuevalue.Bid;
import geniusweb.issuevalue.DiscreteValue;
import geniusweb.issuevalue.Value;
import geniusweb.profile.utilityspace.ProductOfValue;
import geniusweb.profile.utilityspace.SumOfGroupsUtilitySpace;

public class SumOfGroupsLinAdditiveAdapterTest {
	private final ObjectMapper jackson = new ObjectMapper();
	private final Random random = new Random();
	private final SumOfGroupsLinAdditiveAdapter adapter;
	private final SumOfGroupsUtilitySpace groupspace;

	public SumOfGroupsLinAdditiveAdapterTest()
			throws JsonParseException, JsonMappingException, IOException {
		groupspace = jackson.readValue(
				getClass().getResourceAsStream("/computer/computerbuy.json"),
				SumOfGroupsUtilitySpace.class);
		adapter = SumOfGroupsLinAdditiveAdapter.create(groupspace);

	}

	@Test
	public void smokeTest() {
		System.out.println("" + adapter);
	}

	@Test
	public void checkUtility() throws IOException {
		AllBidsList bidspace = new AllBidsList(groupspace.getDomain());
		int NISS = adapter.getDomain().getIssues().size() - 1;

		// compare utility of many random bids
		for (int n = 0; n < 100; n += 1) {
			// space is small, do this simple

			int i = random.nextInt(bidspace.size().intValue());
			// this is bid in the group space.
			Bid groupbid = bidspace.get(BigInteger.valueOf(i));

			Bid linbid = adapter.toLinearBid(groupbid, groupspace);

			assertEquals(groupspace.getUtility(groupbid).floatValue(),
					NISS * adapter.getUtility(linbid).floatValue(), 0.000001);

			assertEquals(groupbid, adapter.toGroupBid(linbid));
		}
	}

	@Test
	public void checkReservationBid() {
		assertEquals(groupspace.getReservationBid(),
				adapter.toGroupBid(adapter.getReservationBid()));
	}

	@Test
	public void testGroupValueSet() {
		final Value val1 = new DiscreteValue("val1");
		final ProductOfValue pov = new ProductOfValue(Arrays.asList(val1));
		final Collection<ProductOfValue> issuevalues = Arrays.asList(pov);
		final GroupValueSet gvs = new GroupValueSet(issuevalues);
		final GroupValueSet gvs1 = new GroupValueSet(issuevalues);

		assertEquals(issuevalues, gvs.getValues());
		assertEquals(BigInteger.ONE, gvs.size());
		assertEquals(pov, gvs.get(BigInteger.ZERO));
		assertEquals(pov, gvs.get(0));
		assertEquals(pov, gvs.iterator().next());
		assertEquals("[" + pov + "]", gvs.toString());

		// poor man's GenericTests, we can't do GenericTests because that's on a
		// class level
		// and we can't make a class level test here.
		assertEquals(gvs, gvs);
		assertEquals(gvs1, gvs);
		assertEquals(gvs.hashCode(), gvs1.hashCode());
	}
}
